@@ -10,10 +10,12 @@ use Rose::DB::Object::Metadata;
 use Rose::Object;
 our @ISA = qw(Rose::Object);
 
+use Rose::DB::Object::Manager;
 use Rose::DB::Object::Constants qw(:all);
-#use Rose::DB::Constants qw(IN_TRANSACTION);
+use Rose::DB::Constants qw(IN_TRANSACTION);
+use Rose::DB::Object::Util qw(row_id);
 
-our $VERSION = '0.075';
+our $VERSION = '0.076';
 
 our $Debug = 0;
 
@@ -113,9 +115,9 @@ sub dbh
 
 sub load
 {
-  my($self) = shift;
+  my($self) = $_[0]; # XXX: Must maintain alias to actual "self" object arg
 
-  my %args = @_;
+  my %args = (self => @_); # faster than @_[1 .. $#_];
 
   my $db  = $self->db  or return 0;
   my $dbh = $self->dbh or return 0;
@@ -161,10 +163,85 @@ sub load
     }
   }
 
-  my $rows = 0;
-
   my $column_names = $meta->column_names;
 
+  #
+  # Handle sub-object load in separate code path
+  #
+
+  if(my $with = $args{'with'})
+  {
+    my $mgr_class = $args{'manager_class'} || 'Rose::DB::Object::Manager';
+    my %query;
+
+    @query{map { "t1.$_" } @key_columns} = @key_values;
+
+    my $objects;
+
+    eval
+    {
+      $objects = 
+        $mgr_class->get_objects(object_class  => ref $self,
+                                query         => [ %query ],
+                                with_objects  => $with,
+                                multi_many_ok => 1)
+          or Carp::confess $mgr_class->error;
+
+      if(@$objects > 1)
+      {
+        die "Found ", @$objects, " objects instead of one";
+      }
+    };
+
+    if($@)
+    {
+      $self->error("load(with => ...) - $@");
+      $self->meta->handle_error($self);
+      return undef;
+    }
+
+    if(@$objects > 0)
+    {
+      # Sneaky init by object replacement
+      $self = $_[0] = $objects->[0];
+
+      # Init by copying attributes (broken; need to do fks and relationships too)
+      #my $methods = $meta->column_mutator_method_names;
+      #my $object  = $objects->[0];
+      #
+      #local $self->{STATE_LOADING()}  = 1;
+      #local $object->{STATE_SAVING()} = 1;
+      #
+      #foreach my $method (@$methods)
+      #{
+      #  $self->$method($object->$method());
+      #}
+    }
+    else
+    {
+      no warnings;
+      $self->error("No such " . ref($self) . ' where ' . 
+                   join(', ', @key_columns) . ' = ' . join(', ', @key_values));
+      $self->{'not_found'} = 1;
+
+      unless($args{'speculative'})
+      {
+        $self->meta->handle_error($self);
+      }
+
+      return 0;
+    }
+
+    $self->{STATE_IN_DB()} = 1;
+    return $self || 1;
+  }
+
+  #
+  # Handle normal load
+  #
+
+  my $rows = 0;
+
   $self->{'not_found'} = 0;
 
   eval
@@ -202,13 +279,24 @@ sub load
 
     if($rows > 0)
     {
+      my $object = (ref $self)->new(db => $self->db);
       my $methods = $meta->column_mutator_method_names_hash;
 
+      # Sneaky init by object replacement
       foreach my $name (@$column_names)
       {
         my $method = $methods->{$name};
-        $self->$method($row{$name});
+        $object->$method($row{$name});
       }
+
+      $self = $_[0] = $object;
+
+      # Init by copying
+      #foreach my $name (@$column_names)
+      #{
+      #  my $method = $methods->{$name};
+      #  $self->$method($row{$name});
+      #}
     }
     else
     {
@@ -237,19 +325,142 @@ sub load
   }
 
   $self->{STATE_IN_DB()} = 1;
-  return 1;
+  return $self || 1;
 }
 
 sub save
 {
   my($self, %args) = @_;
 
-  if(!$args{'insert'} && ($args{'update'} || $self->{STATE_IN_DB()}))
+  # Keep trigger-encumberd code in separate code path
+  if($self->{ON_SAVE_ATTR_NAME()})
   {
-    return shift->update(@_);
-  }
+    my $db = $self->db or return 0;
+    my $ret = $db->begin_work;
 
-  return shift->insert(@_);
+    unless($ret)
+    {
+      $self->error('Could not begin transaction before saving - ' . $db->error);
+      return undef;
+    }
+
+    my $started_new_tx = ($ret == IN_TRANSACTION) ? 0 : 1;
+
+    eval
+    {
+      my $meta = $self->meta;
+      my %did_set;
+
+      #
+      # Do pre-save stuff
+      #
+
+      my $todo = $self->{ON_SAVE_ATTR_NAME()}{'pre'};
+
+      foreach my $fk_name (keys %{$todo->{'fk'}})
+      {
+        my $fk = $meta->foreign_key($fk_name) 
+          or Carp::confess "No foreign key named '$fk_name'";
+
+        my $code   = $todo->{'fk'}{$fk_name}{'set'} or next;
+        my $object = $code->();
+
+        # Account for objects that evaluate to false to due overloading
+        unless($object || ref $object)
+        {
+          die $self->error;
+        }
+
+        # Track which rows were set so we can avoid deleting
+        # them later in the "delete on save" code
+        $did_set{'fk'}{$fk_name}{row_id($object)} = 1;
+      }
+
+      #
+      # Do the actual save
+      #
+
+      if(!$args{'insert'} && ($args{'update'} || $self->{STATE_IN_DB()}))
+      {
+        $ret = shift->update(@_);
+      }
+      else
+      {
+        $ret = shift->insert(@_);
+      }
+
+      #
+      # Do post-save stuff
+      #
+
+      $todo = $self->{ON_SAVE_ATTR_NAME()}{'post'};
+
+      # Foreign keys
+      foreach my $fk_name (keys %{$todo->{'fk'}})
+      {
+        my $fk = $meta->foreign_key($fk_name) 
+          or Carp::confess "No foreign key named '$fk_name'";
+
+        foreach my $item (@{$todo->{'fk'}{$fk_name}{'delete'} || []})
+        {
+          my $code   = $item->{'code'};
+          my $object = $item->{'object'};
+
+          # Don't run the code to delete this object if we just set it above
+          next  if($did_set{'fk'}{$fk_name}{row_id($object)});
+
+          $code->() or die $self->error;
+        }
+      }
+
+      # Relationships
+      foreach my $rel_name (keys %{$todo->{'rel'}})
+      {
+        my $rel = $meta->relationship($rel_name) 
+          or Carp::confess "No relationship named '$rel_name'";
+
+        my $code;
+
+        # Set value(s)
+        if($code  = $todo->{'rel'}{$rel_name}{'set'})
+        {
+          $code->() or die $self->error;
+        }
+
+        # Add value(s)
+        if($code  = $todo->{'rel'}{$rel_name}{'add'})
+        {
+          $code->() or die $self->error;
+        }
+      }
+
+      if($started_new_tx)
+      {
+        $db->commit or die $db->error;
+      }
+    };
+
+    delete $self->{ON_SAVE_ATTR_NAME()};
+
+    if($@)
+    {
+      $self->error($@);
+      $db->rollback or warn $db->error  if($started_new_tx);
+      $self->meta->handle_error($self);
+      return 0;
+    }
+
+    return $ret;
+  }
+  else
+  {
+    if(!$args{'insert'} && ($args{'update'} || $self->{STATE_IN_DB()}))
+    {
+      return shift->update(@_);
+    }
+
+    return shift->insert(@_);
+  }
 }
 
 sub update
@@ -396,7 +607,7 @@ sub update
     return 0;
   }
 
-  return 1;
+  return $self || 1;
 }
 
 sub insert
@@ -534,15 +745,15 @@ sub insert
     return 0;
   }
 
-  return 1;
+  return $self || 1;
 }
 
+my %CASCADE_VALUES = (delete => 'delete', null => 'null', 1 => 'delete');
+
 sub delete
 {
   my($self, %args) = @_;
 
-  my $dbh = $self->dbh or return 0;
-
   my $meta = $self->meta;
 
   my @pk_methods = map { $meta->column_accessor_method_name($_) } $meta->primary_key_column_names;
@@ -556,33 +767,238 @@ sub delete
     return 0;
   }
 
-  eval
+  # Totally separate code path for cascaded delete
+  if(my $cascade = $args{'cascade'})
   {
-    local $self->{STATE_SAVING()} = 1;
-    local $dbh->{'RaiseError'} = 1;
-
-    # Was prepare_cached() but that can't be used across transactions
-    my $sth = $dbh->prepare($meta->delete_sql, $meta->prepare_delete_options);
-
-    $Debug && warn $meta->delete_sql, " - bind params: ", join(', ', @pk_values), "\n";
-    $sth->execute(@pk_values);
-
-    unless($sth->rows > 0)
+    unless(exists $CASCADE_VALUES{$cascade})
     {
-      $self->error("Did not delete " . ref($self) . ' where ' . 
-                   join(', ', @pk_methods) . ' = ' . join(', ', @pk_values));
+      Carp::croak "Illegal value for 'cascade' parameter: '$cascade'.  ",
+                  "Valid values are 'delete', 'null', and '1'";
     }
-  };
 
-  if($@)
-  {
-    $self->error("delete() - $@");
-    $self->meta->handle_error($self);
-    return 0;
+    $cascade = $CASCADE_VALUES{$cascade};
+
+    my $mgr_error_mode = Rose::DB::Object::Manager->error_mode;
+    my($db, $started_new_tx);
+
+    eval
+    {
+      $db = $self->db;
+      my $meta  = $self->meta;
+
+      my $ret = $db->begin_work;
+
+      unless(defined $ret)
+      {
+        die 'Could not begin transaction before deleting with cascade - ',
+            $db->error;
+      }
+
+      $started_new_tx = ($ret == IN_TRANSACTION) ? 0 : 1;
+
+      unless($self->{STATE_IN_DB()})
+      {
+        $self->load 
+          or die "Could not load in preparation for cascading delete: ", 
+                 $self->error;
+      }
+
+      Rose::DB::Object::Manager->error_mode('fatal');
+
+      # Process all the rows for each "... to many" relationship
+      REL: foreach my $relationship ($meta->relationships)
+      {
+        my $rel_type = $relationship->type;
+
+        if($rel_type eq 'one to many')
+        {
+          my $column_map = $relationship->column_map;
+          my @query;
+
+          while(my($local_column, $foreign_column) = each(%$column_map))
+          {
+            my $method = $meta->column_accessor_method_name($local_column);
+            my $value =  $self->$method();
+
+            # XXX: Comment this out to allow null keys
+            next FK  unless(defined $value);
+
+            push(@query, $foreign_column => $value);
+          }
+
+          if($cascade eq 'delete')
+          {
+            Rose::DB::Object::Manager->delete_objects(
+              db           => $db,
+              object_class => $relationship->class,
+              where        => \@query);
+          }
+          elsif($cascade eq 'null')
+          {
+            my %set = map { $_ => undef } values(%$column_map);
+
+            Rose::DB::Object::Manager->update_objects(
+              db           => $db,
+              object_class => $relationship->class,
+              set          => \%set,
+              where        => \@query);        
+          }
+          else { Carp::confess "Illegal cascade value '$cascade' snuck through" }
+        }
+        elsif($rel_type eq 'many to many')
+        {
+          my $map_class  = $relationship->map_class;
+          my $map_from   = $relationship->map_from;
+
+          my $map_from_relationship = 
+            $map_class->meta->foreign_key($map_from)  ||
+            $map_class->meta->relationship($map_from) ||
+            Carp::confess "No foreign key or 'many to one' relationship ",
+                          "named '$map_from' in class $map_class";
+
+          my $key_columns = $map_from_relationship->key_columns;
+          my @query;
+
+          # "Local" here means "local to the mapping table"
+          while(my($local_column, $foreign_column) = each(%$key_columns))
+          {
+            my $method = $meta->column_accessor_method_name($foreign_column);
+            my $value  = $self->$method();
+
+            # XXX: Comment this out to allow null keys
+            next REL  unless(defined $value);
+
+            push(@query, $local_column => $value);
+          }
+
+          if($cascade eq 'delete')
+          {
+            Rose::DB::Object::Manager->delete_objects(
+              db           => $db,
+              object_class => $map_class,
+              where        => \@query);
+          }
+          elsif($cascade eq 'null')
+          {
+            my %set = map { $_ => undef } keys(%$key_columns);
+
+            Rose::DB::Object::Manager->update_objects(
+              db           => $db,
+              object_class => $map_class,
+              set          => \%set,
+              where        => \@query);        
+          }
+          else { Carp::confess "Illegal cascade value '$cascade' snuck through" }
+        }
+      }
+
+      # Delete the object itself
+      my $dbh = $db->dbh or die "Could not get dbh: ", $self->error;
+      local $self->{STATE_SAVING()} = 1;
+      local $dbh->{'RaiseError'} = 1;
+
+      # Was prepare_cached() but that can't be used across transactions
+      my $sth = $dbh->prepare($meta->delete_sql, $meta->prepare_delete_options);
+
+      $Debug && warn $meta->delete_sql, " - bind params: ", join(', ', @pk_values), "\n";
+      $sth->execute(@pk_values);
+
+      unless($sth->rows > 0)
+      {
+        $self->error("Did not delete " . ref($self) . ' where ' . 
+                     join(', ', @pk_methods) . ' = ' . join(', ', @pk_values));
+      }
+
+      # Process all rows referred to by "one to one" foreign keys
+      FK: foreach my $fk ($meta->foreign_keys)
+      {
+        next  unless($fk->relationship_type eq 'one to one');
+
+        my $key_columns = $fk->key_columns;
+        my @query;
+
+        while(my($local_column, $foreign_column) = each(%$key_columns))
+        {
+          my $method = $meta->column_accessor_method_name($local_column);
+          my $value =  $self->$method();
+
+          # XXX: Comment this out to allow null keys
+          next FK  unless(defined $value);
+
+          push(@query, $foreign_column => $value);
+        }
+
+        if($cascade eq 'delete')
+        {
+          Rose::DB::Object::Manager->delete_objects(
+            db           => $db,
+            object_class => $fk->class,
+            where        => \@query);
+        }
+        elsif($cascade eq 'null')
+        {
+          my %set = map { $_ => undef } values(%$key_columns);
+
+          Rose::DB::Object::Manager->update_objects(
+            db           => $db,
+            object_class => $fk->class,
+            set          => \%set,
+            where        => \@query);        
+        }
+        else { Carp::confess "Illegal cascade value '$cascade' snuck through" }
+      }
+
+      if($started_new_tx)
+      {
+        $db->commit or die $db->error;
+      }
+    };
+
+    if($@)
+    {
+      Rose::DB::Object::Manager->error_mode($mgr_error_mode);
+      $self->error("delete() with cascade - $@");
+      $db->rollback  if($db && $started_new_tx);
+      $self->meta->handle_error($self);
+      return 0;
+    }
+
+    Rose::DB::Object::Manager->error_mode($mgr_error_mode);
+    $self->{STATE_IN_DB()} = 0;
+    return 1;
   }
+  else
+  {
+    my $dbh = $self->dbh or return 0;
 
-  $self->{STATE_IN_DB()} = 0;
-  return 1;
+    eval
+    {
+      local $self->{STATE_SAVING()} = 1;
+      local $dbh->{'RaiseError'} = 1;
+
+      # Was prepare_cached() but that can't be used across transactions
+      my $sth = $dbh->prepare($meta->delete_sql, $meta->prepare_delete_options);
+
+      $Debug && warn $meta->delete_sql, " - bind params: ", join(', ', @pk_values), "\n";
+      $sth->execute(@pk_values);
+
+      unless($sth->rows > 0)
+      {
+        $self->error("Did not delete " . ref($self) . ' where ' . 
+                     join(', ', @pk_methods) . ' = ' . join(', ', @pk_values));
+      }
+    };
+
+    if($@)
+    {
+      $self->error("delete() - $@");
+      $self->meta->handle_error($self);
+      return 0;
+    }
+
+    $self->{STATE_IN_DB()} = 0;
+    return 1;
+  }
 }
 
 sub clone
@@ -663,8 +1079,9 @@ Rose::DB::Object - Extensible, high performance RDBMS-OO mapper.
 =head1 SYNOPSIS
 
   ## First, set up your Rose::DB data sources, otherwise you
-  ## won't be able to connect to the database at all!  See 
-  ## the Rose::DB documentation for more information.
+  ## won't be able to connect to the database at all.  See 
+  ## the Rose::DB documentation for more information.  For
+  ## a quick start, run: perldoc Rose::DB::Tutorial
 
   ##
   ## Create classes - two possible approaches:
@@ -800,7 +1217,7 @@ Rose::DB::Object - Extensible, high performance RDBMS-OO mapper.
     {
       type       => 'one to many',
       class      => 'Price',
-      column_map => { id => 'id_product' },
+      column_map => { id => 'product_id' },
     },
   );
 
@@ -885,10 +1302,12 @@ L<Rose::DB::Object> provides the following functions:
 
 =item * Delete a row from the database.
 
-=item * Fetch an object referred to by a foreign key in the current object. (i.e., "one to one" relationships.)
+=item * Fetch an object referred to by a foreign key in the current object. (i.e., "one to one" and "many to one" relationships.)
 
 =item * Fetch multiple objects that refer to the current object, either directly through foreign keys or indirectly through a mapping table.  (i.e., "one to many" and "many to many" relationships.)
 
+=item * Load an object along with "foreign objects" that are related through any of the supported relationship types.
+
 =back
 
 Objects can be loaded based on either a primary key or a unique key.  Since all tables fronted by L<Rose::DB::Object>s must have non-null primary keys, insert, update, and delete operations are done based on the primary key.
@@ -901,9 +1320,13 @@ In addition, its sibling class, L<Rose::DB::Object::Manager>, can do the followi
 
 =item * Iterate over a list of objects, fetching from the database in response to each step of the iterator.
 
-=item * Fetch objects along with "foreign objects" (connected via "one to one" or "one to many" relationships) in a single query by automatically generating the appropriate SQL join(s).
+=item * Fetch objects along with "foreign objects" (related through any of the supported relationship types) in a single query by automatically generating the appropriate SQL join(s).
 
-=item * Count the number of objects that match a given query.
+=item * Count the number of objects that match a complex query.
+
+=item * Update objects that match a complex query.
+
+=item * Delete objects that match a complex query.
 
 =back
 
@@ -971,10 +1394,30 @@ Override this method in your subclass in order to use a different default data s
 
 Returns the L<DBI> database handle contained in L<db|/db>.
 
-=item B<delete>
+=item B<delete [PARAMS]>
 
 Delete the row represented by the current object.  The object must have been previously loaded from the database (or must otherwise have a defined primary key value) in order to be deleted.  Returns true if the row was deleted or did not exist, false otherwise.
 
+PARAMS are optional name/value pairs.  Valid PARAMS are:
+
+=over 4
+
+=item C<cascade TYPE>
+
+Also process related rows.  TYPE must be "delete", "null", or "1".  The value "1" is an alias for "delete".  Passing an illegal TYPE value will cause a fatal error.
+
+For each "one to many" relationship, all of the rows in the foreign ("many") table that reference the current object ("one") will be deleted in "delete" mode, or will have the column(s) that reference the current object set to NULL in "null" mode.
+
+For each "many to many" relationship, all of the rows in the "mapping table" that reference the current object will deleted in "delete" mode, or will have the columns that reference the two tables that the mapping table maps between set to NULL in "null" mode.
+
+For each "one to one" relationship or foreign key with a "one to one" L<relationship type|Rose::DB::Object::Metadata::ForeignKey/relationship_type>, all of the rows in the foreign table that reference the current object will deleted in "delete" mode, or will have the column(s) that reference the current object set to NULL in "null" mode.
+
+In all modes, if the L<db|/db> is not currently in a transaction (i.e., if L<AutoCommit|Rose::DB/autocommit> is turned off), a new transaction is started.  If any part of the cascaded delete fails, the transaction is rolled back.
+
+=back
+
+The cascaded delete feature described above plays it safe by only deleting rows that are not referenced by any other rows (according to the metadata provided by each L<Rose::DB::Object>-derived class).  I B<strongly recommend> that you implement "cascaded delete" in the database itself, rather than using this feature.  It will undoubtedly be faster and more robust than doing it "client-side."  You may also want to cascade only to certain tables, or otherwise deviate from the "safe" plan.  If your database supports automatic cascaded delete and/or triggers, please consider using thse features.
+
 =item B<error>
 
 Returns the text message associated with the last error that occurred.
@@ -983,9 +1426,49 @@ Returns the text message associated with the last error that occurred.
 
 Load a row from the database table, initializing the object with the values from that row.  An object can be loaded based on either a primary key or a unique key.
 
-Returns true if the row was loaded successfully, undef if the row could not be loaded due to an error, or zero (0) if the row does not exist.
+Returns true if the row was loaded successfully, undef if the row could not be loaded due to an error, or zero (0) if the row does not exist.  The true value returned on success will be the object itself.  If the object L<overload>s its boolean value such that it is not true, then a true value will be returned instead of the object itself.
 
-PARAMS are optional name/value pairs.  If the parameter C<speculative> is passed with a true value, and if the load failed because the row was L<not found|/not_found>, then the L<error_mode|Rose::DB::Object::Metadata/error_mode> setting is ignored and zero (0) is returned.
+PARAMS are optional name/value pairs.  Valid PARAMS are:
+
+=over 4
+
+=item C<speculative BOOL>
+
+If this parameter is passed with a true value, and if the load failed because the row was L<not found|/not_found>, then the L<error_mode|Rose::DB::Object::Metadata/error_mode> setting is ignored and zero (0) is returned.
+
+=item C<with OBJECTS>
+
+Load the object and the specified "foreign objects" simultaneously.  OBJECTS should be a reference to an array of L<foreign key|Rose::DB::Object::Metadata/foreign_keys> or L<relationship|Rose::DB::Object::Metadata/relationships> names.
+
+=back
+
+B<SUBCLASS NOTE:> If you are going to override the L<load|/load> method in your subclass, you I<must> pass an I<alias to the actual object> as the first argument to the method, rather than passing a copy of the object reference.  Example:
+
+    # This is the CORRECT way to override load() while still
+    # calling the base class version of the method.
+    sub load
+    {
+      my $self = $_[0]; # Copy, no shift
+
+      ... # Do your stuff
+
+      shift->SUPER::load(@_); # Call superclass
+    }
+
+Now here's the wrong way:
+
+    # This is the WRONG way to override load() while still
+    # calling the base class version of the method.
+    sub load
+    {
+      my $self = shift; # WRONG!  The alias to original object is now lost!
+
+      ... # Do your stuff
+
+      $self->SUPER::load(@_); # This won't work right!
+    }
+
+This requirement exists in order to preserve some sneaky object-replacement optimizations in the base class implementation of L<load|/load>.  At some point, those optimizations may change or go away.  But if you follow these guidelines, your code will continue to work no matter what.
 
 =item B<not_found>
 
@@ -1017,7 +1500,7 @@ If set to a true value, then an update is attempted, regardless of whether or no
 
 It is an error to pass both the C<insert> and C<update> parameters in a single call.
 
-Returns true if the row was inserted or updated successfully, false otherwise.
+Returns true if the row was inserted or updated successfully, false otherwise.  The true value returned on success will be the object itself.  If the object L<overload>s its boolean value such that it is not true, then a true value will be returned instead of the object itself.
 
 If an insert was performed and the primary key is a single column that supports auto-generated values, then the object accessor for the primary key column will contain the auto-generated value.
 
@@ -1112,6 +1595,20 @@ Here is a list of method names reserved by the L<Rose::DB::Object> API.  If you
 
 Note that not all of these methods are public.  These methods do not suddenly become public just because you now know their names!  Remember the stated policy of the L<Rose> web application framework: if a method is not documented, it does not exist.  (And no, the list of method names above does not constitute "documentation")
 
+=head1 DEVELOPMENT POLICY
+
+The L<Rose development policy|Rose/"DEVELOPMENT POLICY"> applies to this, and all C<Rose::*> modules.  Please install L<Rose> from CPAN and then run "C<perldoc Rose>" for more information.
+
+=head1 SUPPORT
+
+Any L<Rose::DB::Objects> questions or problems can be posted to the L<Rose::DB::Object> mailing list.  To subscribe to the list or view the archives, go here:
+
+L<http://lists.sourceforge.net/lists/listinfo/rose-db-object>
+
+Although the mailing list is the preferred support mechanism, you can also email the author (see below) or file bugs using the CPAN bug tracking system:
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Rose-DB-Object>
+
 =head1 AUTHOR
 
 John C. Siracusa (siracusa@mindspring.com)
