@@ -9,10 +9,10 @@ use Rose::DB::Object::QueryBuilder qw(build_select);
 
 use Rose::DB::Object::Constants qw(STATE_LOADING STATE_IN_DB);
 
-# XXX: Should be a value that is unlikely to exist in a primary key column
+# XXX: A value that is unlikely to exist in a primary key column value
 use constant PK_JOIN => "\0\2,\3\0";
 
-our $VERSION = '0.062';
+our $VERSION = '0.07';
 
 our $Debug = 0;
 
@@ -64,13 +64,15 @@ sub handle_error
 
 sub object_class { }
 
+sub default_manager_method_types { qw(objects iterator count delete update) }
+
 sub make_manager_methods
 {
   my($class) = shift;
 
   if(@_ == 1)
   {
-    @_ = (methods => { $_[0] => [ qw(objects iterator count) ] });
+    @_ = (methods => { $_[0] => [ $class->default_manager_method_types ] });
   }
   else
   {
@@ -115,7 +117,10 @@ sub make_manager_methods
                   "You must supply one or the other";
     }
 
-    $args{'methods'} = { $args{'base_name'} => [ qw(objects iterator count) ] };
+    $args{'methods'} = 
+    { 
+      $args{'base_name'} => [ $class->default_manager_method_types ] 
+    };
   }
   elsif($args{'base_name'})
   {
@@ -198,6 +203,42 @@ sub make_manager_methods
             @_, return_iterator => 1, object_class => $object_class)
         };
       }
+      elsif($type eq 'delete')
+      {
+        my $method_name = 
+          $have_full_name ? $name : "${target_class}::delete_$name";
+
+        foreach my $class ($target_class, $class_invocant)
+        {
+          my $method = "${class}::delete_$name";
+          Carp::croak "A $method method already exists"
+            if(defined &{$method});
+        }
+
+        *{$method_name} = sub
+        {
+          shift;
+          $class_invocant->delete_objects(@_, object_class => $object_class);
+        };
+      }
+      elsif($type eq 'update')
+      {
+        my $method_name = 
+          $have_full_name ? $name : "${target_class}::update_$name";
+
+        foreach my $class ($target_class, $class_invocant)
+        {
+          my $method = "${class}::update_$name";
+          Carp::croak "A $method method already exists"
+            if(defined &{$method});
+        }
+
+        *{$method_name} = sub
+        {
+          shift;
+          $class_invocant->update_objects(@_, object_class => $object_class);
+        };
+      }
       else
       {
         Carp::croak "Invalid method type: $type";
@@ -224,10 +265,11 @@ sub get_objects
   my $return_sql      = delete $args{'return_sql'};
   my $return_iterator = delete $args{'return_iterator'};
   my $object_class    = delete $args{'object_class'} or Carp::croak "Missing object class argument";
-  my $require_objects = delete $args{'require_objects'};
-  my $with_objects    = delete $args{'with_objects'};
-  my $skip_first      = delete $args{'skip_first'} || 0;
   my $count_only      = delete $args{'count_only'};
+  my $require_objects = delete $args{'require_objects'};
+  my $with_objects    = !$count_only ? delete $args{'with_objects'} : undef;
+  my $skip_first      = delete $args{'skip_first'} || 0;
+
 
   my $db  = delete $args{'db'} || $object_class->init_db;
   my $dbh = delete $args{'dbh'};
@@ -267,7 +309,15 @@ sub get_objects
     }
     else
     {
-      $with_objects = [ $with_objects ]  unless(ref $with_objects);
+      if(ref $with_objects) # copy argument (shallow copy)
+      {
+        $with_objects = [ @$with_objects ];
+      }
+      else
+      {
+        $with_objects = [ $with_objects ];
+      }
+
       $num_with_objects = @$with_objects;
       %with_objects = map { $_ => 1 } @$with_objects;
     }
@@ -275,7 +325,14 @@ sub get_objects
 
   if($require_objects)
   {
-    $require_objects = [ $require_objects ]  unless(ref $require_objects);
+    if(ref $require_objects) # copy argument (shallow copy)
+    {
+      $require_objects = [ @$require_objects ];
+    }
+    else
+    {
+      $require_objects = [ $require_objects ];
+    }
 
     $num_required_objects = @$require_objects;
     %required_object = map { $_ => 1 } @$require_objects;
@@ -303,7 +360,7 @@ sub get_objects
   my %methods = ($tables[0] => scalar $meta->column_mutator_method_names);
   my @classes = ($object_class);
   my %meta    = ($object_class => $meta);
-  my(@joins, @subobject_methods);
+  my(@joins, @subobject_methods, $clauses);
 
   my $handle_dups = 0;
   my @has_dups;
@@ -314,7 +371,8 @@ sub get_objects
 
   if($with_objects)
   {
-    my $clauses = $args{'clauses'} ||= [];
+    # Copy clauses arg
+    $clauses = $args{'clauses'} ? [ @{$args{'clauses'}} ] : [];
 
     my $i = 1;
 
@@ -366,7 +424,7 @@ sub get_objects
 
     unless($args{'multi_many_ok'})
     {
-      if(scalar(grep { $_ } @has_dups) > 1)
+      if(scalar(grep { defined $_ } @has_dups) > 1)
       {
         Carp::carp
           qq(WARNING: Fetching sub-objects via more than one ),
@@ -411,8 +469,13 @@ sub get_objects
         $classes{$tables[-1]} = $ft_class;
         $methods{$tables[-1]} = $ft_meta->column_mutator_method_names;
 
-        $subobject_methods[$i - 1] = $rel->method_name('get_set')
-          or Carp::confess "No 'get_set' method found for relationship '$name' in class $class";
+        $subobject_methods[$i - 1] = 
+          $rel->method_name('get_set') ||
+          $rel->method_name('get_set_now') ||
+          $rel->method_name('get_set_on_save') ||
+          Carp::confess "No 'get_set', 'get_set_now', or 'get_set_on_save' ",
+                        "method found for relationship '$name' in class ",
+                        "$class";
 
         # Reset each() iterator
         keys(%$ft_columns);
@@ -575,8 +638,13 @@ sub get_objects
         # Increase again for foreign table.
         $i++;
 
-        $subobject_methods[$i - 1] = $rel->method_name('get_set')
-          or Carp::confess "No 'get_set' method found for relationship '$name' in class $class";
+        $subobject_methods[$i - 1] =
+          $rel->method_name('get_set') ||
+          $rel->method_name('get_set_now') ||
+          $rel->method_name('get_set_on_save') ||
+          Carp::confess "No 'get_set', 'get_set_now', or 'get_set_on_save' ",
+                        "method found for relationship '$name' in class ",
+                        "$class";
 
         # Reset each() iterator
         keys(%$ft_columns);
@@ -626,6 +694,8 @@ sub get_objects
         Carp::croak "Don't know how to auto-join relationship '$name' of type '$rel_type'";
       }
     }
+
+    $args{'clauses'} = $clauses; # restore clauses arg
   }
 
   if($count_only)
@@ -633,17 +703,48 @@ sub get_objects
     delete $args{'limit'};
     delete $args{'offset'};
     delete $args{'sort_by'};
+          
+    my($sql, $bind);
 
-    my($sql, $bind) =
-      build_select(dbh     => $dbh,
-                   select  => 'COUNT(*)',
-                   tables  => \@tables,
-                   columns => \%columns,
-                   classes => \%classes,
-                   meta    => \%meta,
-                   db      => $db,
-                   pretty  => $Debug,
-                   %args);
+    my $use_distinct = 0; # Do we have to use DISTINCT to count?
+
+    if($require_objects)
+    {
+      foreach my $name (@$require_objects)
+      {
+        # Ignore error here since it'll be caught and handled later anyway
+        my $key = 
+          $meta->foreign_key($name) || $meta->relationship($name) || next;
+
+        my $rel_type = $key->type;
+
+        if(index($key->type, 'many') >= 0)
+        {
+          $use_distinct = 1;
+          last;
+        }
+      }
+    }
+
+    BUILD_SQL:
+    {
+      my $select = $use_distinct ? 'COUNT(DISTINCT ' .
+        join(', ', map { "t1.$_" } $meta->primary_key_column_names) . ')' :
+        'COUNT(*)';
+
+      local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+
+      ($sql, $bind) =
+        build_select(dbh     => $dbh,
+                     select  => $select,
+                     tables  => \@tables,
+                     columns => \%columns,
+                     classes => \%classes,
+                     meta    => \%meta,
+                     db      => $db,
+                     pretty  => $Debug,
+                     %args);
+    }
 
     if($return_sql)
     {
@@ -678,7 +779,7 @@ sub get_objects
   # Alter sort_by SQL, replacing table names with aliases.  This is to
   # prevent databases like Postgres from "adding missing FROM clause"s.
   # See: http://sql-info.de/postgresql/postgres-gotchas.html#1_5
-  if(my $sort = $args{'sort_by'})
+  if((my $sort = $args{'sort_by'}) && $num_subtables > 0)
   {
     my $i = 0;
 
@@ -691,7 +792,7 @@ sub get_objects
     $args{'sort_by'} = $sort;
   }
 
-  if($args{'offset'})
+  if(defined $args{'offset'})
   {
     Carp::croak "Offset argument is invalid without a limit argument"
       unless($args{'limit'} || $manual_limit);
@@ -720,16 +821,23 @@ sub get_objects
 
   my($count, @objects, $iterator);
 
-  my($sql, $bind) =
-    build_select(dbh     => $dbh,
-                 tables  => \@tables,
-                 columns => \%columns,
-                 classes => \%classes,
-                 joins   => \@joins,
-                 meta    => \%meta,
-                 db      => $db,
-                 pretty  => $Debug,
-                 %args);
+  my($sql, $bind);
+
+  BUILD_SQL:
+  {
+    local $Carp::CarpLevel = $Carp::CarpLevel + 1;
+
+    ($sql, $bind) =
+      build_select(dbh     => $dbh,
+                   tables  => \@tables,
+                   columns => \%columns,
+                   classes => \%classes,
+                   joins   => \@joins,
+                   meta    => \%meta,
+                   db      => $db,
+                   pretty  => $Debug,
+                   %args);
+  }
 
   if($return_sql)
   {
@@ -829,6 +937,7 @@ sub get_objects
                         {
                           # Mapping tables will have no subobject methods, so skip them
                           my $method = $subobject_methods[$i] or next;
+                          local $last_object->{STATE_LOADING()} = 1;
                           $last_object->$method($sub_objects[$i]);
                         }
                       }
@@ -884,6 +993,7 @@ sub get_objects
                     else # Otherwise, just assign it
                     {
                       my $method = $subobject_methods[$i];
+                      local $object->{STATE_LOADING()} = 1;
                       $object->$method($subobject);
                     }
                   }
@@ -926,6 +1036,7 @@ sub get_objects
                     {
                       # Mapping tables will have no subobject methods, so skip them
                       my $method = $subobject_methods[$i] or next;
+                      local $last_object->{STATE_LOADING()} = 1;
                       $last_object->$method($sub_objects[$i]);
                     }
                   }
@@ -938,6 +1049,8 @@ sub get_objects
                   $sth = undef;
                   last ROW;
                 }
+                
+                last ROW;
               }
             };
 
@@ -948,8 +1061,11 @@ sub get_objects
               return undef;
             }
 
+            @objects = ()  if($skip_first);
+
             if(@objects)
             {
+              no warnings; # undef count okay
               if($manual_limit && $self->{'_count'} == $manual_limit)
               {
                 $self->total($self->{'_count'});
@@ -1022,7 +1138,7 @@ sub get_objects
               return undef;
             }
 
-            return $object;
+            return $skip_first ? undef : $object;
           });
         }
       }
@@ -1121,6 +1237,7 @@ sub get_objects
                 {
                   # Mapping tables will have no subobject methods, so skip them
                   my $method = $subobject_methods[$i] or next;
+                  local $last_object->{STATE_LOADING()} = 1;
                   $last_object->$method($sub_objects[$i]);
                 }
               }
@@ -1128,7 +1245,7 @@ sub get_objects
               # Add the object to the final list of objects that we'll return
               push(@objects, $last_object);
 
-              if($manual_limit && @objects == $manual_limit)
+              if(!$skip_first && $manual_limit && @objects == $manual_limit)
               {
                 last ROW;
               }
@@ -1177,6 +1294,7 @@ sub get_objects
             else # Otherwise, just assign it
             {
               my $method = $subobject_methods[$i];
+              local $object->{STATE_LOADING()} = 1;
               $object->$method($subobject);
             }
           }
@@ -1194,7 +1312,7 @@ sub get_objects
 
         # Handle the left-over "last object" that needs to be finished and
         # added to the final list of objects to return.
-        if($last_object)
+        if($last_object && !$skip_first)
         {
           foreach my $i (1 .. $num_subtables)
           {
@@ -1202,6 +1320,7 @@ sub get_objects
             {
               # Mapping tables will have no subobject methods, so skip them
               my $method = $subobject_methods[$i] or next;
+              local $last_object->{STATE_LOADING()} = 1;
               $last_object->$method($sub_objects[$i]);
             }
           }
@@ -1211,6 +1330,8 @@ sub get_objects
             push(@objects, $last_object);
           }
         }
+        
+        @objects = ()  if($skip_first);
       }
       else # simple sub-objects case: nothing worse than one-to-one relationships
       {
@@ -1308,7 +1429,177 @@ sub _map_action
 }
 
 sub save_objects   { shift->_map_action('save', @_)   }
-sub delete_objects { shift->_map_action('delete', @_) }
+
+sub delete_objects
+{
+  my($class, %args) = @_;
+
+  $class->error(undef);
+
+  $args{'query'} = delete $args{'where'};
+
+  my $object_class = $args{'object_class'} or Carp::croak "Missing object class argument";
+
+  unless(($args{'query'} && @{$args{'query'}}) || delete $args{'all'})
+  {
+    Carp::croak "$class - Refusing to delete all rows from the table '",
+                $object_class->meta->fq_table_sql, "' without an explict ",
+                "'all => 1' parameter";
+  }
+
+  if($args{'query'} && @{$args{'query'}} && $args{'all'})
+  {
+    Carp::croak "Illegal use of the 'where' and 'all' parameters in the same call";
+  }
+
+  my $db  = $args{'db'} || $object_class->init_db;
+  my $dbh = $args{'dbh'};
+  my $dbh_retained = 0;
+
+  unless($dbh)
+  {
+    unless($dbh = $db->retain_dbh)
+    {
+      $class->error($db->error);
+      $class->handle_error($class);
+      return undef;
+    }
+
+    $args{'dbh'} = $dbh;
+    $dbh_retained = 1;
+  }
+
+  my $meta = $object_class->meta;
+
+  # Yes, I'm re-using get_objects() code like crazy, and often
+  # in weird ways.  Shhhh, it's a secret.
+
+  my($where, $bind) = 
+    $class->get_objects(%args, return_sql => 1, where_only => 1);
+
+  my $sql = 'DELETE FROM ' . $meta->fq_table_sql .
+            ($where ? " WHERE\n$where" : '');
+
+  my $count;
+
+  eval
+  {
+    local $dbh->{'RaiseError'} = 1;  
+    $Debug && warn "$sql - bind params: ", join(', ', @$bind), "\n";
+
+    my $sth = $dbh->prepare($sql, $meta->prepare_bulk_delete_options) 
+      or die $dbh->errstr;
+
+    $sth->execute(@$bind);
+    $count = $sth->rows || 0;
+  };
+
+  if($@)
+  {
+    $class->error("delete_objects() - $@");
+    $class->handle_error($class);
+    return undef;
+  }
+
+  return $count;
+}
+
+sub update_objects
+{
+  my($class, %args) = @_;
+
+  $class->error(undef);
+
+  my $object_class = $args{'object_class'} 
+    or Carp::croak "Missing object class argument";
+
+  unless(($args{'where'} && @{$args{'where'}}) || delete $args{'all'})
+  {
+    Carp::croak "$class - Refusing to update all rows in the table '",
+                $object_class->meta->fq_table_sql, "' without an explict ",
+                "'all => 1' parameter";
+  }
+
+  if($args{'where'} && @{$args{'where'}} && $args{'all'})
+  {
+    Carp::croak "Illegal use of the 'where' and 'all' parameters in the same call";
+  }
+
+  my $where = delete $args{'where'};
+  my $set   = delete $args{'set'} 
+    or Carp::croak "Missing requires 'set' parameter";
+
+  $set = [ %$set ]  if(ref $set eq 'HASH');
+
+  my $db  = $args{'db'} || $object_class->init_db;
+  my $dbh = $args{'dbh'};
+  my $dbh_retained = 0;
+
+  unless($dbh)
+  {
+    unless($dbh = $db->retain_dbh)
+    {
+      $class->error($db->error);
+      $class->handle_error($class);
+      return undef;
+    }
+
+    $args{'dbh'} = $dbh;
+    $dbh_retained = 1;
+  }
+
+  my $meta = $object_class->meta;
+
+  # Yes, I'm re-using get_objects() code like crazy, and often
+  # in weird ways.  Shhhh, it's a secret.
+
+  $args{'query'} = $set;
+
+  my($set_sql, $set_bind) = 
+    $class->get_objects(%args, return_sql => 1, where_only => 1, logic => ',', set => 1);
+
+  my $sql;
+
+  my $where_bind = [];
+
+  if($args{'query'} = $where)
+  {
+    my $where_sql;
+
+    ($where_sql, $where_bind) = 
+      $class->get_objects(%args, return_sql => 1, where_only => 1);
+
+    $sql = 'UPDATE ' . $meta->fq_table_sql . 
+           "\nSET\n$set_sql\nWHERE\n$where_sql";
+  }
+  else
+  {
+    $sql = 'UPDATE ' . $meta->fq_table_sql . "\nSET\n$set_sql";
+  }
+
+  my $count;
+
+  eval
+  {
+    local $dbh->{'RaiseError'} = 1;  
+    $Debug && warn "$sql\n";
+
+    my $sth = $dbh->prepare($sql, $meta->prepare_bulk_update_options) 
+      or die $dbh->errstr;
+
+    $sth->execute(@$set_bind, @$where_bind);
+    $count = $sth->rows || 0;
+  };
+
+  if($@)
+  {
+    $class->error("update_objects() - $@");
+    $class->handle_error($class);
+    return undef;
+  }
+
+  return $count;
+}
 
 1;
 
@@ -1389,6 +1680,7 @@ Rose::DB::Object::Manager - Fetch multiple Rose::DB::Object-derived objects from
     name        => { type => 'varchar', length => 255 },
     description => { type => 'text' },
     category_id => { type => 'int' },
+    region_num  => { type => 'int' },
 
     status => 
     {
@@ -1466,6 +1758,16 @@ Rose::DB::Object::Manager - Fetch multiple Rose::DB::Object-derived objects from
   # {
   #   shift->get_objects_count(@_, object_class => 'Product');
   # }
+  #
+  # sub update_products
+  # {
+  #   shift->update_objects(@_, object_class => 'Product');
+  # }
+  #
+  # sub delete_products
+  # {
+  #   shift->delete_objects(@_, object_class => 'Product');
+  # }
 
   ...
 
@@ -1585,16 +1887,81 @@ Rose::DB::Object::Manager - Fetch multiple Rose::DB::Object-derived objects from
     }
   }
 
+  #
+  # Update objects
+  #
+
+  $num_rows_updated =
+    Product::Manager->update_products(
+      set =>
+      {
+        end_date   => DateTime->now,
+        region_num => { sql => 'region_num * -1' }
+        status     => 'defunct',
+      },
+      where =>
+      [
+        start_date => { lt => '1/1/1980' },
+        status     => [ 'active', 'pending' ],
+      ]);
+
+  #
+  # Delete objects
+  #
+
+  $num_rows_deleted =
+    Product::Manager->delete_products(
+      where =>
+      [
+        status  => [ 'stale', 'old' ],
+        name    => { like => 'Wax%' }
+        or =>
+        [
+          start_date => { gt => '2008-12-30' },
+          end_date   => { gt => 'now' },
+        ],
+      ]);
+
 =head1 DESCRIPTION
 
 C<Rose::DB::Object::Manager> is a base class for classes that select rows from tables fronted by L<Rose::DB::Object>-derived classes.  Each row in the table(s) queried is converted into the equivalent L<Rose::DB::Object>-derived object.
 
-Class methods are provided for fetching objects all at once, one at a time through the use of an iterator, or just getting the object count.  Subclasses are expected to create syntactically pleasing wrappers for C<Rose::DB::Object::Manager> class methods.  A very minimal example is shown in the L<synopsis|/SYNOPSIS> above.
+Class methods are provided for fetching objects all at once, one at a time through the use of an iterator, or just getting the object count.  Subclasses are expected to create syntactically pleasing wrappers for C<Rose::DB::Object::Manager> class methods, either manually or with the L<make_manager_methods|/make_manager_methods> method.  A very minimal example is shown in the L<synopsis|/SYNOPSIS> above.
 
 =head1 CLASS METHODS
 
 =over 4
 
+=item B<delete_objects [PARAMS]>
+
+Delete rows from a table fronted by a L<Rose::DB::Object>-derived class based on PARAMS, where PARAMS are name/value pairs.  Returns the number of rows deleted, or undef if there was an error.
+
+Valid parameters are:
+
+=over 4
+
+=item C<all BOOL>
+
+If set to a true value, this parameter indicates an explicit request to delete all rows from the table.  If both the C<all> and the C<where> parameters are passed, a fatal error will occur.
+
+=item C<db DB>
+
+A L<Rose::DB>-derived object used to access the database.  If omitted, one will be created by calling the L<init_db|Rose::DB::Object/init_db> object method of the C<object_class>.
+
+=item C<object_class CLASS>
+
+The name of the L<Rose::DB::Object>-derived class that fronts the table from which rows are to be deleted.  This parameter is required; a fatal error will occur if it is omitted.
+
+=item C<where PARAMS>
+
+The query parameters, passed as a reference to an array of name/value pairs.  These PARAMS are used to formulate the "where" clause of the SQL query that is used to delete the rows from the table.  Arbitrarily nested boolean logic is supported.
+
+For the complete list of valid parameter names and values, see the documentation for the C<query> parameter of the L<build_select|Rose::DB::Object::QueryBuilder/build_select> function in the L<Rose::DB::Object::QueryBuilder> module.
+
+If this parameter is omitted, this method will refuse to delete all rows from the table and a fatal error will occur.  To delete all rows from a table, you must pass the C<all> parameter with a true value.  If both the C<all> and the C<where> parameters are passed, a fatal error will occur.
+
+=back
+
 =item B<error>
 
 Returns the text message associated with the last error, or false if there was no error.
@@ -1639,7 +2006,7 @@ In all cases, the class's C<error> attribute will also contain the error message
 
 =item B<get_objects [PARAMS]>
 
-Get L<Rose::DB::Object>-derived objects based on PARAMS, where PARAMS are name/value pairs.  Returns a  reference to a (possibly empty) array in scalar context, a list of objects in list context, or undef if there was an error.  
+Get L<Rose::DB::Object>-derived objects based on PARAMS, where PARAMS are name/value pairs.  Returns a reference to a (possibly empty) array in scalar context, a list of objects in list context, or undef if there was an error.  
 
 Note that naively calling this method in list context may result in a list containing a single undef element if there was an error.  Example:
 
@@ -1677,9 +2044,9 @@ The "products" table is "t1" since it's the primary table--the table behind the
     # Table aliases in the comments
     $products = 
       Product::Manager->get_products(
-                           # t4
+                           # t5
         require_objects => [ 'category' ],
-                           # t1            t2, t3
+                           # t2            t3, t4
         with_objects    => [ 'code_names', 'colors' ],
         multi_many_ok   => 1,
         query           => [ status => 'defunct' ],
@@ -1713,7 +2080,7 @@ A reference to a hash of name/value pairs to be passed to the constructor of eac
 
 =item C<object_class CLASS>
 
-The class name of the L<Rose::DB::Object>-derived objects to be fetched.  This parameter is required; a fatal error will occur if it is omitted.
+The name of the L<Rose::DB::Object>-derived objects to be fetched.  This parameter is required; a fatal error will occur if it is omitted.
 
 =item C<offset NUM>
 
@@ -1751,7 +2118,7 @@ B<Note:> the C<with_objects> list currently cannot be used to simultaneously fet
 
 The query parameters, passed as a reference to an array of name/value pairs.  These PARAMS are used to formulate the "where" clause of the SQL query that, in turn, is used to fetch the objects from the database.  Arbitrarily nested boolean logic is supported.
 
-For the complete list of valid parameter names and values, see the L<build_select|Rose::DB::Object::QueryBuilder/build_select> function of the L<Rose::DB::Object::QueryBuilder> module.
+For the complete list of valid parameter names and values, see the documentation for the C<query> parameter of the L<build_select|Rose::DB::Object::QueryBuilder/build_select> function in the L<Rose::DB::Object::QueryBuilder> module.
 
 =back
 
@@ -1783,7 +2150,7 @@ The class of the L<Rose::DB::Object>-derived objects to be fetched or counted.
 
 =item * B<base name> or B<method name>
 
-The base name is a string used as the basis of the method names.  For example, the base name "products" would be used to create methods named "get_B<products>", "get_B<products>_count", and "get_B<products>_iterator"
+The base name is a string used as the basis of the method names.  For example, the base name "products" would be used to create methods named "get_B<products>", "get_B<products>_count", "get_B<products>_iterator", "delete_B<products>", and "update_B<products>".
 
 In the absence of a base name, an explicit method name may be provided instead.  The method name will be used as is.
 
@@ -1796,6 +2163,8 @@ The types of methods that should be generated.  Each method type is a wrapper fo
     objects     get_objects()
     iterator    get_objects_iterator()
     count       get_objects_count()
+    delete      delete_objects()
+    update      update_objects()
 
 =item * B<target class>
 
@@ -1968,6 +2337,16 @@ Finally, sometimes you don't want or need to use L<make_manager_methods|/make_ma
     shift->get_objects_count(object_class => 'Product', @_);
   }
 
+  sub delete_products
+  {
+    shift->delete_objects(object_class => 'Product', @_);
+  }
+
+  sub update_products
+  {
+    shift->update_objects(object_class => 'Product', @_);
+  }
+
 Of course, these methods will all look very similar in each L<Rose::DB::Object::Manager>-derived class.  Creating these identically structured methods is exactly what L<make_manager_methods|/make_manager_methods> automates for you.  
 
 But sometimes you want to customize these methods, in which case the "longhand" technique above becomes essential.  For example, imagine that we want to extend the code in the L<synopsis|/SYNOPSIS>, adding support for a C<with_categories> parameter to the C<get_products()> method.  
@@ -1994,6 +2373,75 @@ Here we've coerced the caller-friendly C<with_categories> boolean flag parameter
 
 This is the typical evolution of an object manager method.  It starts out as being auto-generated by L<make_manager_methods|/make_manager_methods>, then becomes customized as new arguments are added.
 
+=item B<update_objects [PARAMS]>
+
+Update rows in a table fronted by a L<Rose::DB::Object>-derived class based on PARAMS, where PARAMS are name/value pairs.  Returns the number of rows updated, or undef if there was an error.
+
+Valid parameters are:
+
+=over 4
+
+=item C<all BOOL>
+
+If set to a true value, this parameter indicates an explicit request to update all rows in the table.  If both the C<all> and the C<where> parameters are passed, a fatal error will occur.
+
+=item C<db DB>
+
+A L<Rose::DB>-derived object used to access the database.  If omitted, one will be created by calling the L<init_db|Rose::DB::Object/init_db> object method of the C<object_class>.
+
+=item C<object_class CLASS>
+
+The class name of the L<Rose::DB::Object>-derived class that fronts the table whose rows will to be updated.  This parameter is required; a fatal error will occur if it is omitted.
+
+=item C<set PARAMS>
+
+The names and values of the columns to be updated.  PARAMS should be a reference to a hash.  Each key of the hash should be a column name or column get/set method name.  If a value is a simple scalar, then it is passed through the get/set method that services the column before being incorporated into the SQL query.
+
+If a value is a reference to a hash, then it must contain a single key named "sql" and a corresponding value that will be incorporated into the SQL query as-is.  For example, this method call:
+
+  $num_rows_updated =
+    Product::Manager->update_products(
+      set =>
+      {
+        end_date   => DateTime->now,
+        region_num => { sql => 'region_num * -1' }
+        status     => 'defunct',
+      },
+      where =>
+      [
+        status  => [ 'stale', 'old' ],
+        name    => { like => 'Wax%' }
+        or =>
+        [
+          start_date => { gt => '2008-12-30' },
+          end_date   => { gt => 'now' },
+        ],
+      ]);
+
+would produce the an SQL statement something like this (depending on the database vendor, and assuming the current date was September 20th, 2005):
+
+    UPDATE products SET
+      end_date   = '2005-09-20',
+      region_num = region_num * -1,
+      status     = 'defunct'
+    WHERE
+      status IN ('stale', 'old') AND
+      name LIKE 'Wax%' AND
+      (
+        start_date > '2008-12-30' OR
+        end_date   > '2005-09-20'
+      )
+
+=item C<where PARAMS>
+
+The query parameters, passed as a reference to an array of name/value pairs.  These PARAMS are used to formulate the "where" clause of the SQL query that is used to update the rows in the table.  Arbitrarily nested boolean logic is supported.
+
+For the complete list of valid parameter names and values, see the documentation for the C<query> parameter of the L<build_select|Rose::DB::Object::QueryBuilder/build_select> function in the L<Rose::DB::Object::QueryBuilder> module.
+
+If this parameter is omitted, this method will refuse to update all rows in the table and a fatal error will occur.  To update all rows in a table, you must pass the C<all> parameter with a true value.  If both the C<all> and the C<where> parameters are passed, a fatal error will occur.
+
+=back
+
 =back
 
 =head1 AUTHOR
